@@ -20,8 +20,15 @@
 
 -- imports
 import("core.base.option")
+import("core.base.hashset")
 
 -- main entry
 function main()
-    import(option.get("format") or "local")()
+    local format = option.get("format") or "local"
+    local maps = hashset.from({"local", "remote", "oldpkg"})
+    if maps:has(format) then
+        import(format)()
+    else
+        raise("invalid package format: %s", format or "missing")
+    end
 end
