@@ -22,6 +22,7 @@
 import("core.base.option")
 import("core.project.config")
 import("core.base.semver")
+import("core.base.hashset")
 import("core.tool.linker")
 import("core.tool.compiler")
 import("core.language.language")
@@ -167,10 +168,14 @@ function get_clang_target(toolchain)
 end
 
 -- get clang target flags
+-- @see https://github.com/xmake-io/xmake/issues/7271
 function get_clang_target_flags(toolchain)
-    local target = get_clang_target(toolchain)
-    if target then
-        return "--target=" .. target
+    local target
+    if toolchain:is_cross() or toolchain:is_plat("windows") then
+        target = get_clang_target(toolchain)
+        if target then
+            return "--target=" .. target
+        end
     end
     if toolchain:is_arch("x86_64", "x64") then
         return "-m64"
@@ -555,3 +560,21 @@ function get_sanitizer_flags(target, opt)
     end
     return result
 end
+
+-- check if the file is a c++ header file extension
+function is_cxx_headerext(extension)
+    -- prioritize .h* extensions to filter out most cases quickly
+    if extension:startswith(".h") then
+        return true
+    end
+
+    local headerexts = _g.headerexts
+    if not headerexts then
+        local other_header_extensions = {
+            ".inl", ".ipp", ".tcc", ".tpl", ".inc"
+        }
+        headerexts = hashset.from(other_header_extensions)
+        _g.headerexts = headerexts
+    end
+    return headerexts:has(extension) or false
+end
